use crate::{Context, Handler};
use actix_web::{post, web, HttpResponse, Responder};
use serde::Deserialize;
use serde_json::json;
use verax::{AccountId, AnyAmount, Status};

#[derive(Deserialize)]
pub struct AccountWithAmount {
    pub account: AccountId,
    #[serde(flatten)]
    pub amount: AnyAmount,
}

#[derive(Deserialize)]
pub struct Transaction {
    pub debit: Vec<AccountWithAmount>,
    pub credit: Vec<AccountWithAmount>,
    pub memo: String,
    pub status: Status,
}

#[async_trait::async_trait]
impl Handler for Transaction {
    type Ok = verax::Transaction;
    type Err = verax::Error;

    async fn handle(self, ledger: &Context) -> Result<Self::Ok, Self::Err> {
        let from = self
            .debit
            .into_iter()
            .map(|x| x.amount.try_into().map(|amount| (x.account, amount)))
            .collect::<Result<Vec<_>, _>>()?;

        let to = self
            .credit
            .into_iter()
            .map(|x| x.amount.try_into().map(|amount| (x.account, amount)))
            .collect::<Result<Vec<_>, _>>()?;

        ledger
            .ledger
            .new_transaction(self.memo, self.status, from, to)
            .await
    }
}

#[post("/tx")]
async fn handler(item: web::Json<Transaction>, ledger: web::Data<Context>) -> impl Responder {
    match item.into_inner().handle(&ledger).await {
        Ok(tx) => HttpResponse::Accepted().json(tx),
        Err(err) => {
            HttpResponse::InternalServerError().json(json!({ "text": err.to_string(), "err": err}))
        }
    }
}
